@@ -1,19 +1,19 @@
 ;;; -*- byte-compile-dynamic: t -*-
-;;; mucs-ccl.el --- for MULE-UCS basic ccl configurations
+;;; mucs-ccl.el --- Mule-UCS CCL manager.(MUCS-CCL)
 
 ;; Copyright (C) 1997-2000 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, 
-;;           character set, coding-system, ISO10646, Unicode
+;;           character set, coding-system, CCL
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -42,44 +42,6 @@
 ;;; 5 ... produce the table structures that are used by ExCCL.
 ;;;
 
-;;;
-;;; To check invalid code
-;;;
-
-(defun mucs-ccl-if (statement true-block &optional false-block)
-  )
-(put 'mucs-ccl-if 'lisp-indent-function 2)
-
-(defun mucs-ccl-if-invalid (invalid-part &optional valid-part)
-  (if valid-part
-      `((if (r0 == ,mucs-invalid-code)
-	    (,@invalid-part)
-	  (,@valid-part)))
-    `((if (r0 == ,mucs-invalid-code)
-	  (,@invalid-part)))))
-
-(defun mucs-ccl-if-valid (valid-part)
-  `((if (r0 != ,mucs-invalid-code)
-	(,@valid-part))))
-
-(defun mucs-ccl-if-invalid-repeat ()
-  (mucs-ccl-if-invalid '((repeat))))
-
-;;;
-;;; Common I/O interface.
-;;;
-
-(defun mucs-ccl-read (type serialize)
-  (setq mucs-current-type type)
-  (mucs-type-get-serialize-method type serialize t))
-
-(defun mucs-ccl-write (serialize)
-  (if mucs-current-type
-      (mucs-ccl-if-valid
-       (mucs-type-get-serialize-method
-	mucs-current-type serialize nil))
-    (error "Current TYPE is not set.  Cannot write.")))
-
 ;;;
 ;;; some simple ccl codes.
 ;;;
@@ -132,14 +94,14 @@
 (mucs-type-register-serialization
  'char-1
  'emacs-mule
- '((write r0)) ;;;; umm....
+ '(quote (((write r0)))) ;;;; umm....
  'none)
 
 (mucs-type-register-serialization
  'char-packed-1
  'emacs-mule
- (mucs-ccl-write-char-packed-1)
- (mucs-ccl-read-char-packed-1))
+ (quote `(,(mucs-ccl-write-char-packed-1)))
+ (quote `(,(mucs-ccl-read-char-packed-1))))
 
 (mucs-type-register-conversion
  'char-1
@@ -218,13 +180,13 @@
 (mucs-type-register-serialization
  'char-1
  'emacs-mule-dos
- (mucs-ccl-write-char-1-dos)
+ (quote `(,(mucs-ccl-write-char-1-dos)))
  'none)
 
 (mucs-type-register-serialization
  'char-packed-1
  'emacs-mule-dos
- (mucs-ccl-write-char-packed-1-dos)
+ (quote `(,(mucs-ccl-write-char-packed-1-dos)))
  'none)
 
 
@@ -273,8 +235,7 @@
   (cons 'myo (list
 	      (cons 'ccl nil)
 	      (cons 'elisp nil)
-	      (cons 'table-set nil)))
-  )
+	      (cons 'table-set nil))))
 
 (defsubst mucs-ccl-myo-from-ccl (ccl)
   (cons 'myo (list (list 'ccl ccl)
@@ -297,6 +258,28 @@ MYO1 and MYO2 may be destructed."
 	    (setq alist2 (delq elem2 alist2)))))
     (nconc myo1 alist2)))
 
+(defsubst mucs-ccl-myo-append-safe (myo1 myo2)
+  "Returns concatinated MYO1 and MYO2.
+MYO1 and MYO2 is never destructed."
+  (let ((alist1 (cdr myo1))
+	(alist2 (copy-sequence (cdr myo2)))
+	result
+	elem1 elem2)
+    (while (setq elem1 (car alist1))
+      (setq elem2 (assq (car elem1) alist2)
+	    alist1 (cdr alist1))
+      (if elem2
+	  (progn
+	    (setq result (cons
+			  (append elem1
+				  (cdr elem2))
+			  result))
+	    (setq alist2 (delq elem2 alist2)))
+	(setq result (cons elem1
+			   result))))
+    (cons 'myo
+	  (nconc result alist2))))
+
 
 (defsubst mucs-ccl-myo-add-elisp (elisp myo)
   (let ((slot (assq 'elisp (cdr myo))))
@@ -374,9 +357,9 @@ MYO1 and MYO2 may be destructed."
     (function generate-table-set-registration-program)
     (mucs-ccl-myo-get-table-set myo))))
 
-;;
-;; API.
-;;
+;;;
+;;; MUCS Conversion Engine API.
+;;;
 
 (defsubst mucs-convert (conversion object)
   (let ((ccl-object
@@ -403,9 +386,6 @@ MYO1 and MYO2 may be destructed."
 	   execute-vector
 	 (aref execute-vector 0))))))
 
-(defun mucs-setup-defun-convsersion (conv def)
-)
-
 (defun mucs-setup-conversion (conv def)
   (let ((convtype (mucs-conversion-get-conv-type conv))
 	(mucs-current-conversion conv)
@@ -465,4 +445,76 @@ MYO1 and MYO2 may be destructed."
     (mucs-conversion-set-program-and-compiled-code
      conv program compiled-ccl)))
 
+;;;
+;;; MUCS-CCL standard library.
+;;;
+
+(defun mucs-ccl-if (expression true-part &optional false-part)
+  (let (true-part-myo
+	false-part-myo
+	result-myo)
+    (setq true-part-myo (mucs-ccl-bind-program (eval true-part)))
+    (if false-part
+	(progn
+	  (setq false-part-myo (mucs-ccl-bind-program (eval false-part))
+		result-myo (mucs-ccl-myo-append-safe
+			    true-part-myo
+			    false-part-myo))
+	  (mucs-ccl-myo-set-ccl
+	   `((if ,expression
+		 ,(mucs-ccl-myo-get-ccl true-part-myo)
+	       ,(mucs-ccl-myo-get-ccl false-part-myo)))
+	   result-myo))
+      (mucs-ccl-myo-set-ccl
+       `((if ,expression
+	     ,(mucs-ccl-myo-get-ccl true-part-myo)))
+       true-part-myo))))
+
+(put 'mucs-ccl-if 'lisp-indent-function 2)
+
+(defun mucs-ccl-if-invalid (invalid-part &optional valid-part)
+  (let (invalid-part-myo valid-part-myo result-myo)
+    (setq invalid-part-myo (mucs-ccl-bind-program (eval invalid-part)))
+    (if valid-part
+	(progn
+	  (setq valid-part-myo (mucs-ccl-bind-program (eval valid-part))
+		result-myo (mucs-ccl-myo-append-safe
+			    invalid-part-myo
+			    valid-part-myo))
+	  (mucs-ccl-myo-set-ccl
+	   `((if (r0 == ,mucs-invalid-code)
+		 ,(mucs-ccl-myo-get-ccl invalid-part-myo)
+	       ,(mucs-ccl-myo-get-ccl valid-part-myo)))
+	   result-myo))
+      (mucs-ccl-myo-set-ccl
+       `((if (r0 == ,mucs-invalid-code)
+	     ,(mucs-ccl-myo-get-ccl invalid-part-myo)))
+       invalid-part-myo))))
+
+(defun mucs-ccl-if-valid (valid-part)
+  (let ((valid-part-myo (mucs-ccl-bind-program (eval valid-part))))
+    (mucs-ccl-myo-set-ccl
+     `((if (r0 != ,mucs-invalid-code)
+	   ,(mucs-ccl-myo-get-ccl valid-part-myo)))
+     valid-part-myo)))
+
+(defun mucs-ccl-if-invalid-repeat ()
+  (mucs-ccl-if-invalid '(quote (((repeat))))))
+
+;;
+;; Common I/O interface.
+;;
+
+(defun mucs-ccl-read (type serialize)
+  (setq mucs-current-type type)
+  (mucs-ccl-bind-program
+   (eval (mucs-type-get-serialize-method type serialize t))))
+
+(defun mucs-ccl-write (serialize)
+  (if mucs-current-type
+      (mucs-ccl-if-valid
+       (mucs-type-get-serialize-method
+	mucs-current-type serialize nil))
+    (error "Current TYPE is not set.  Cannot write.")))
+
 (provide 'mucs-ccl)
