@@ -4,16 +4,16 @@
 ;; Keywords: CCL, mule, multilingual, 
 ;;           character set, coding-system, ISO10646, Unicode, IPA
 
-;; Copyright (C) 1999 Miyashita Hisashi
+;; Copyright (C) 1999-2000 Miyashita Hisashi
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -23,7 +23,8 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-(put 'ipa 'unicode 'ipa-vs-unicode-assoc)
+(put 'ipa 'unicode-assoc
+     'ipa-vs-unicode-assoc)
 
 (defvar
   ipa-vs-unicode-assoc
