@@ -1,18 +1,18 @@
 ;;; -*- coding: iso-2022-7bit  -*-
 ;;; tae.el --- Translation rule And Encoding compiler(TAE:$BL/(B):-)
 
-;; Copyright (C) 1997-1999 Miyashita Hisashi
+;; Copyright (C) 1997-2000 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, encode, CCL
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -30,6 +30,7 @@
 (require 'mucs-type)
 (require 'tbl-mg)
 
+;;; primitive data for translation.
 
 (defun tae-project-all-translation-vector (val)
   (vector t
@@ -37,24 +38,39 @@
 	      mucs-invalid-code val)
 	  0 (mucs-max-code)))
 
-;(register-code-conversion-map
-; 'tae-cease-translation-table
-; (vector t -1 0 (mucs-max-code)))
+;;; generate unique symbol
 
+(defvar tae-embedded-name-obarry
+  (make-vector 37 0))
+
+(defun tae-generate-unique-symbol-for-embed (name type decodep)
+  (intern
+   (format "%s-%s-%s"
+	   (symbol-name name)
+	   (symbol-name type)
+	   (if decodep
+	       "decode"
+	     "encode"))
+   tae-embedded-name-obarry))
+
+;; table set manager
 
 (defvar tae-translation-table-set-index 1)
 (defsubst tae-generate-translation-table-set-name
-  (tr-name &optional decodep index)
+  (tr-name &optional type decodep index)
     "Return a name of a symbol used to register a table-set,
 which is managed by TAE."
     (if (null index)
 	(setq index tae-translation-table-set-index
 	      tae-translation-table-set-index (1+ index)))
-    (format "%s-tae-table-set-%s-%d"
+    (format "%s-tts-%s-%s-%d"
 	    (symbol-name tr-name)
+	    (symbol-name type)
 	    (if decodep "decode" "encode")
 	    index))
 
+(defvar tae-current-compiling-translation nil)
+
 ;;
 ;;
 ;; Interface to declare translation
@@ -87,6 +103,8 @@ which is managed by TAE."
 ;;                   Produced TAE messages for decode.
 ;;               tae-table-set:
 ;;                   Generated table-sets.
+;;                   This is an A-list whose key is
+;;                   (TYPE . (ENCODE-TABLE-SET . DECODE-TABLE-SET))
 
 (defvar tae-internal-translation-property
   '(tae-translation
@@ -95,12 +113,18 @@ which is managed by TAE."
     tae-reduced-translation-for-decode
     tae-products-for-encode
     tae-products-for-decode
-    tae-table-set))
+    tae-table-set
+;;
+    tae-dependent-translations
+    tae-dependent-conversions))
 
 (defsubst tae-declared-translation-p (name)
   (and (symbolp name)
        (get name 'tae-translation)))
 
+(defsubst tae-translation-dynamic-p (name)
+  (get name 'tae-dynamic-translation))
+
 (defun tae-copy-declared-translation (dest src)
   (mapcar
    (lambda (x)
@@ -113,25 +137,43 @@ which is managed by TAE."
       (table-set-add-reference (car table-sets))
       (setq table-sets (cdr table-sets)))))
 
+(defun tae-translation-add-table-set (name table-set type decodep)
+  (let* ((alist (get name 'tae-table-set))
+	 (maybe (cdr (assq type alist))))
+    (if (and maybe
+	     (if decodep
+		 (cdr maybe)
+	       (car maybe)))
+	(error "Translation %S has already have table-set:%S(TYPE:%S, DECODEP:%S)."
+	       name maybe type decodep))
+    (if maybe
+	(if decodep
+	    (setcdr maybe table-set)
+	  (setcar maybe table-set))
+      (put name 'tae-table-set
+	   (cons (cons type
+		       (if decodep
+			   (cons nil table-set)
+			 (cons table-set nil)))
+		 alist)))))
+
+(defsubst tae-translation-get-table-set (name type decodep)
+  (if decodep
+      (cdr (cdr (assq type (get name 'tae-table-set))))
+    (car (cdr (assq type (get name 'tae-table-set))))))
 
 (defun tae-initialize-translation (name)
   (set name nil)
 
-  ;;; Remove reference all generated table-sets.
+  ;;; Remove references to all generated table-sets.
   (mapcar
    (lambda (x)
-     (table-set-remove-reference x))
-   (get name 'tae-table-set))
-
-  ;;; any properties of internal use must be set to nil.
-  (mapcar
-   (lambda (x)
-     (put name x nil))
-   tae-internal-translation-property))
-
-(defun tae-translation-add-table-set (name table-set)
-  (put name 'tae-table-set
-       (cons table-set (get name 'tae-table-set))))
+     (setq x (cdr x))
+     (if (car x)
+	 (table-set-remove-reference (car x)))
+     (if (cdr x)
+	 (table-set-remove-reference (cdr x))))
+   (get name 'tae-table-set)))
 
 (defsubst tae-get-translation-produced-products (translation type decodep)
   (cdr (assq type
@@ -152,6 +194,52 @@ which is managed by TAE."
 	 (cons (cons type value)
 	       slots))))
 
+;;; registered translation manager
+
+(defun tae-register-translation (name)
+  (let ((trlist (get mucs-current-package
+		     'tae-registered-translation)))
+    (if (not (memq name trlist))
+	(progn
+	  (setq trlist (cons name trlist))
+	  (put mucs-current-package
+	       'tae-registered-translation
+	       trlist)))))
+
+(defun tae-retrieve-registered-translation-list ()
+  (get mucs-current-package
+       'tae-registered-translation))
+
+(defun tae-register-dependent-translation (dep-name)
+  (let ((trlist (get tae-current-compiling-translation
+		     'tae-dependent-translations)))
+    (if (not (memq dep-name trlist))
+	(put tae-current-compiling-translation
+	     'tae-dependent-translations
+	     (cons dep-name trlist)))))
+
+(defun tae-register-conversion (name type decodep conversion)
+  (let ((trlist (cons name
+		      (get name 'tae-dependent-translations)))
+	marksym
+	slot
+	convlist
+	cur)
+;    (message "MS:%S->%S" marksym trlist)
+    (while (setq cur (car trlist))
+      (setq convlist (get cur 'tae-dependent-conversions)
+	    trlist (cdr trlist))
+;      (message "TR:%S NM:%S CONV:%S" cur name conversion)
+      (setq marksym (tae-generate-unique-symbol-for-embed
+		     name type decodep))
+      (if (not (and (setq slot (assq conversion convlist))
+		    (eq marksym (nth 1 slot))))
+	  (put cur 'tae-dependent-conversions
+	       (cons (list conversion name marksym type decodep)
+		     convlist))))))
+
+;;; translation declaration.
+
 (defun tae-declare-translation (name translation &optional dynamicp)
   "Declare translation.
 NAME must be a symbol to distinguish TRANSLATION.
@@ -392,52 +480,60 @@ When decodep is non-nil, normalize for decording."
   (setq tr-elem (copy-sequence tr-elem))
   (cond ((tae-tr-elem-assoc-p tr-elem)
 	 (let ((curll (tae-get-tr-elem-assoc tr-elem))
+	       key-type relative-op
 	       curel alist-copy all-slot)
 	   
 	   ;;; copy alist and remove 'all and invalid slot.
 	   (if decodep
-	     (while 
-		 (and (setq curel (car curll))
-		      (cond ((eq (cdr curel) 'all)
-			     (setq all-slot curel)
-			     nil)
-			    ;; skip the slot
-			    ((or (eq (car curel) 'all)
-				 (eq (cdr curel) 'invalid))
-			     (setq curll (cdr curll)))
-			    (t
-			     (setq alist-copy (cons (cons (car curel)
-							  (cdr curel))
-						    alist-copy))
-			     (setq curll (cdr curll))))))
-	     (while 
-		 (and (setq curel (car curll))
-		      (cond ((eq (car curel) 'all)
-			     (setq all-slot curel)
-			     nil)
-			    ;; skip the slot
-			    ((or (eq (cdr curel) 'all)
-				 (eq (car curel) 'invalid))
-			     (setq curll (cdr curll)))
-			    (t
-			     (setq alist-copy (cons (cons (car curel)
-							  (cdr curel))
-						    alist-copy))
-			     (setq curll (cdr curll)))))))
+	       (progn
+		 (setq key-type (cdr (tae-get-tr-elem-type-spec tr-elem))
+		       relative-op (mucs-type-get-relative-op key-type))
+		 (while 
+		     (and (setq curel (car curll))
+			  (cond ((eq (cdr curel) 'all)
+				 (setq all-slot curel)
+				 nil)
+				;; skip the slot
+				((or (eq (car curel) 'all)
+				     (eq (cdr curel) 'invalid))
+				 (setq curll (cdr curll)))
+				(t
+				 (setq alist-copy (cons (cons (car curel)
+							      (cdr curel))
+							alist-copy))
+				 (setq curll (cdr curll)))))))
+		 (setq key-type (cdr (tae-get-tr-elem-type-spec tr-elem))
+		       relative-op (mucs-type-get-relative-op key-type))
+		 (while 
+		     (and (setq curel (car curll))
+			  (cond ((eq (car curel) 'all)
+				 (setq all-slot curel)
+				 nil)
+				;; skip the slot
+				((or (eq (cdr curel) 'all)
+				     (eq (car curel) 'invalid))
+				 (setq curll (cdr curll)))
+				(t
+				 (setq alist-copy (cons (cons (car curel)
+							      (cdr curel))
+							alist-copy))
+				 (setq curll (cdr curll)))))))
 
-	 (tae-set-tr-elem-assoc
-	  tr-elem
-	  (nconc
-	   (sort alist-copy
-		 (if decodep
-		     (lambda (x y) (< (cdr x) (cdr y)))
-		   (lambda (x y) (< (car x) (car y)))))))
-	 (if all-slot
-	     (tae-set-tr-elem-all-key tr-elem
-				      (if decodep
-					  (car all-slot)
-					(cdr all-slot))))
-	 ))
+	   (tae-set-tr-elem-assoc
+	    tr-elem
+	    (nconc
+	     (sort alist-copy
+		   (if decodep
+		       (lambda (x y) (funcall relative-op
+					      (cdr x) (cdr y)))
+		     (lambda (x y) (funcall relative-op
+					    (car x) (car y)))))))
+	   (if all-slot
+	       (tae-set-tr-elem-all-key tr-elem
+					(if decodep
+					    (car all-slot)
+					  (cdr all-slot))))
+	   ))
 
 	((tae-tr-elem-elisp-p tr-elem)
 	 (let ((range-list (tae-check-and-normalize-range
@@ -492,19 +588,24 @@ When DECODEP is t, reduce for decodeing translation."
 		   (let* ((alist1 (tae-get-tr-elem-assoc tr1))
 			  (alist2 (tae-get-tr-elem-assoc tr2))
 			  (cur-ll2 alist2)
-			  cur-ll1
-			  cur-el1 cur-el2
-			  result)
+			  (type-spec (tae-get-tr-elem-type-spec tr1))
+			  (key-type (if decodep (cdr type-spec)
+				      (car type-spec)))
+			  (relative-op (mucs-type-get-relative-op key-type))
+			  (equal-op (mucs-type-get-equal-op key-type))
+			  cur-ll1 cur-el1 cur-el2 result)
 		     (setq cur-el1 (car alist1)
 			   cur-ll1 (cons nil alist1))
 		     (while (setq cur-el2 (car cur-ll2))
 		       (while 
 			   (progn
 			     (cond ((if decodep
-					(> (cdr cur-el1)
-					   (cdr cur-el2))
-				      (> (car cur-el1)
-					 (car cur-el2)))
+					(funcall relative-op
+						 (cdr cur-el2)
+						 (cdr cur-el1))
+					(funcall relative-op
+						 (car cur-el2)
+						 (car cur-el1)))
 				    (if (null (car cur-ll1))
 					(setq alist1
 					      (cons cur-el2
@@ -514,10 +615,12 @@ When DECODEP is t, reduce for decodeing translation."
 						    (cdr cur-ll1))))
 				    nil)
 				   ((if decodep
-					(= (cdr cur-el1)
-					   (cdr cur-el2))
-				      (= (car cur-el1)
-					 (car cur-el2)))
+					(funcall equal-op
+						 (cdr cur-el1)
+						 (cdr cur-el2))
+				      (funcall equal-op
+					       (car cur-el1)
+					       (car cur-el2)))
 				    nil)
 				   (t
 				    (setq cur-ll1 (cdr cur-ll1))
@@ -587,18 +690,24 @@ When DECODEP is t, reduce for decodeing translation."
 			    (alist1 (tae-get-tr-elem-assoc tr1))
 			    (alist2 (tae-get-tr-elem-assoc tr2))
 			    (cur-ll2 alist2)
-			    cur-ll1
-			    cur-el1 cur-el2)
+			    (type-spec (tae-get-tr-elem-type-spec tr1))
+			    (key-type (if decodep (cdr type-spec)
+					(car type-spec)))
+			    (relative-op (mucs-type-get-relative-op key-type))
+			    (equal-op (mucs-type-get-equal-op key-type))
+			    cur-ll1 cur-el1 cur-el2)
 		       (while (setq cur-el2 (car cur-ll2))
 			 (setq cur-el1 (car alist1)
 			       cur-ll1 (cons nil alist1))
 			 (while 
 			     (progn
 			       (cond ((if decodep
-					  (= (cdr cur-el1)
-					     (cdr cur-el2))
-					(= (car cur-el1)
-					   (car cur-el2)))
+					  (funcall equal-op
+						   (cdr cur-el1)
+						   (cdr cur-el2))
+					(funcall equal-op
+						 (car cur-el1)
+						 (car cur-el2)))
 				      (setq result-alist
 					    (cons cur-el2 result-alist))
 				      nil)
@@ -653,16 +762,21 @@ When DECODEP is t, reduce for decodeing translation."
 			  (alist1 (tae-get-tr-elem-assoc tr1))
 			  (alist2 (tae-get-tr-elem-assoc tr2))
 			  (cur-ll2 alist2)
-			  cur-ll1
-			  cur-el1 cur-el2
+			  (type-spec (tae-get-tr-elem-type-spec tr1))
+			  (key-type (if decodep (cdr type-spec)
+				      (car type-spec)))
+			  (relative-op (mucs-type-get-relative-op key-type))
+			  (equal-op (mucs-type-get-equal-op key-type))
+			  cur-ll1 cur-el1 cur-el2
 			  tr-new-all-key)
 		     (while (setq cur-el2 (car cur-ll2))
 		       (setq cur-el1 (car alist1)
 			     cur-ll1 (cons nil alist1))
 		       (while 
 			   (progn
-			     (cond ((= (cdr cur-el1)
-				       (car cur-el2))
+			     (cond ((funcall equal-op
+					     (cdr cur-el1)
+					     (car cur-el2))
 				    (setq result-alist
 					  (cons
 					   (cons (car cur-el1)
@@ -689,7 +803,9 @@ When DECODEP is t, reduce for decodeing translation."
 					    (car cur-el2))
 				      result-alist))
 			     (if (and (null tr-new-all-key)
-				      (= tr1-all-key (car cur-el2)))
+				      (funcall equal-op
+					       tr1-all-key
+					       (car cur-el2)))
 				 (setq tr-new-all-key (cdr cur-el2)))
 				 ))
 		       (setq cur-ll2 (cdr cur-ll2)))
@@ -774,50 +890,65 @@ according to it, rearrange the elements of UNIT."
 	(if (tae-tr-elem-normalized-p cur-el decodep)
 	    nil
 	  (setcar cur-ll (tae-normalize-translation cur-el decodep)))
-      (setcar cur-ll (tae-reduce-translation cur-el decodep)))
+      (setcar cur-ll (tae-reduce-translation-internal cur-el decodep)))
     (setq cur-ll (cdr cur-ll))))
   unit)
 
-(defun tae-reduce-translation (translation &optional decodep)
+(defun tae-reduce-translation-internal (translation &optional decodep)
+  (cond ((tae-declared-translation-p translation)
+	 translation)
+	((tae-tr-elem-p translation)
+	 (or (tae-tr-elem-normalized-p translation decodep)
+	     (setq translation
+		   (tae-normalize-translation translation decodep)))
+	 translation)
+	(t
+	 (let ((op (car translation))
+	       (args (copy-sequence (cdr translation))))
+	   (cond ((eq op '|)
+		  (setq args (tae-normalize-reduction-unit args decodep))
+		  (setq args (tae-reduce-OR-translations args decodep))
+		  (if (= (length args) 1)
+		      (car args)
+		    (cons '| args)))
+		 ((eq op '&)
+		  (setq args (tae-normalize-reduction-unit args decodep))
+		  (setq args (tae-reduce-AND-translations args decodep))
+		  (if (= (length args) 1)
+		      (car args)
+		    (cons '& args)))
+		 ((eq op 'c)
+		  (setq args (tae-normalize-reduction-unit args decodep))
+		  (setq args 
+			(tae-normalize-reduction-unit
+			 (tae-reduce-composite-translations args decodep)
+			 decodep))
+		  (if (= (length args) 1)
+		      (car args)
+		    (cons 'c args)))
+		 ((eq op 'ct)
+		  (setq args (tae-normalize-reduction-unit args decodep))
+		  (setq args 
+			(tae-normalize-reduction-unit
+			 (tae-reduce-composite-transparent-translations
+			  args decodep)
+			 decodep))
+		  (if (= (length args) 1)
+		      (car args)
+		    (cons 'ct args)))
+		 (t
+		  (error "TAE have not supported OP:`%S' yet!!" op)))))))
+
+(defsubst tae-reduce-translation (name &optional decodep)
   "Reduce translation."
-  (if (or (tae-declared-translation-p translation)
-	  (tae-tr-elem-p translation))
-      translation
-    (let ((op (car translation))
-	  (args (copy-sequence (cdr translation))))
-      (cond ((eq op '|)
-	     (setq args (tae-normalize-reduction-unit args decodep))
-	     (setq args (tae-reduce-OR-translations args decodep))
-	     (if (= (length args) 1)
-		 (car args)
-	       (cons '| args)))
-	    ((eq op '&)
-	     (setq args (tae-normalize-reduction-unit args decodep))
-	     (setq args (tae-reduce-AND-translations args decodep))
-	     (if (= (length args) 1)
-		 (car args)
-	       (cons '& args)))
-	    ((eq op 'c)
-	     (setq args (tae-normalize-reduction-unit args decodep))
-	     (setq args 
-		   (tae-normalize-reduction-unit
-		    (tae-reduce-composite-translations args decodep)
-		    decodep))
-	     (if (= (length args) 1)
-		 (car args)
-	       (cons 'c args)))
-	    ((eq op 'ct)
-	     (setq args (tae-normalize-reduction-unit args decodep))
-	     (setq args 
-		   (tae-normalize-reduction-unit
-		    (tae-reduce-composite-transparent-translations
-		     args decodep)
-		    decodep))
-	     (if (= (length args) 1)
-		 (car args)
-	       (cons 'ct args)))
-	    (t
-	     (error "TAE have not supported OP:`%S' yet!!" op))))))
+  (let ((sym (if decodep
+		 'tae-reduced-translation-for-decode
+	       'tae-reduced-translation-for-encode)))
+    (or (get name sym)
+	(put name sym
+	     (tae-reduce-translation-internal
+	      (symbol-value name)
+	      decodep)))))
 
 ;; TAE message
 ;;   Not yet fixed :-P.
@@ -864,14 +995,6 @@ according to it, rearrange the elements of UNIT."
 	  message)
       (cons (cons 'tables tables) message))))
 
-(defsubst tae-message-set-table-set (message table-set)
-  (let ((slot (assq 'table-set message)))
-    (if slot
-	(progn
-	  (setcdr slot type)
-	  message)
-      (cons (cons 'new-type type) message))))
-
 (defsubst tae-message-set-new-type (message type)
   (let ((slot (assq 'new-type message)))
     (if slot
@@ -880,11 +1003,19 @@ according to it, rearrange the elements of UNIT."
 	  message)
       (cons (cons 'new-type type) message))))
 
-(defun tae-generate-new-table-set (tables name decodep)
-  (let ((table-set
-	 (intern
-	  (tae-generate-translation-table-set-name name decodep))))
-    (tae-translation-add-table-set name table-set)
+(defun tae-generate-new-table-set (tables name type decodep)
+  (let (table-set)
+    (while
+	(progn
+	  (setq table-set
+		;;; It is useful for debugging
+		;;; to intern generated symbols.
+		(make-symbol
+		 (tae-generate-translation-table-set-name
+		  name type decodep)))
+	  (table-set-p table-set)))
+    (tae-translation-add-table-set
+     name table-set type decodep)
     (define-table-set table-set tables)
     table-set))
 
@@ -900,12 +1031,11 @@ according to it, rearrange the elements of UNIT."
 	 (to-func (mucs-type-get-ccl-representation to-type))
 	 (conv-func (if (eq type from-type)
 			'identity
-		      (mucs-type-get-conversion from-type type)))
+		      (mucs-type-get-conversion-force
+		       from-type type)))
 	 (tr-elem-all-key (tae-get-tr-elem-all-key tr-elem))
 	 conv-slot
 	 result)
-    (if (null conv-func)
-	(error "TYPE:%S cannot be converted to %S" from-type type))
     (setq result
 	  (cond ((tae-tr-elem-assoc-p tr-elem)
 		 (if (not (tae-tr-elem-normalized-p tr-elem decodep))
@@ -963,12 +1093,13 @@ according to it, rearrange the elements of UNIT."
 	(setq to-type new-type)
 	(setq results
 	      (tae-message-set-new-type results to-type)))
-      ;;; operate 'table-set key.
-      (if (setq table-set-new-slot (assq 'table-set result))
+      ;;; operate table-set.
+      (if (and (tae-declared-translation-p tr)
+	       (setq table-set-new-slot
+		     (tae-translation-get-table-set tr type decodep)))
 	  (progn
 	    (setq results (tae-message-add-tables
-			   results (list (cdr table-set-new-slot))))
-	    (setq result (delq table-set-new-slot result))))
+			   results (list table-set-new-slot)))))
       ;;; operate 'tables key.
       (if (setq tables-new-slot (assq 'tables result))
 	  (progn
@@ -988,40 +1119,52 @@ according to it, rearrange the elements of UNIT."
 (defun tae-compile-composite-transparent-operation (translations type decodep)
   (error "Not supported yet:"))
 
-(defsubst tae-get-compiled-products-internal (translation type decodep)
+(defun tae-get-compiled-products-internal (translation type decodep)
   (let (op args)
     (if (consp translation)
 	(setq op (car translation)
 	      args (cdr translation)))
     (cond ((tae-declared-translation-p translation)
-	    (tae-get-compiled-products translation type decodep))
-	   ((tae-tr-elem-p translation)
-	    (tae-compile-translation-element translation type decodep))
-	   ((eq op '|)
-	    (tae-compile-OR-operation args type decodep))
-	   ((eq op '&)
-	    (tae-compile-AND-operation args type decodep))
-	   ((eq op 'c)
-	    (tae-compile-composite-operation args type decodep))
-	   ((eq op 'ct)
-	    (tae-compile-composite-transparent-operation args type decodep))
-	   (t
-	    (error "TAE have not supported OP:`%S' yet!!" op)))))
+	   ;;; If this translation is dynamic,
+	   ;;; all referred translations are used to
+	   ;;; dynamic modification.
+	   ;(if (tae-translation-dynamic-p
+	   ;   tae-current-compiling-translation)
+	   ;   (tae-register-dependent-translation
+	   ;   translation nil))
+	   ;;; If the referred transation is dynamic,
+	   ;;; this translation must be also modified by
+	   ;;; dynamic modification.
+	   ;(if (tae-translation-dynamic-p translation)
+	   ;    (tae-register-dependent-translation
+	   ;    translation t))
+	   (tae-register-dependent-translation
+	    translation)
+	   (tae-get-compiled-products translation type decodep))
+	  ((tae-tr-elem-p translation)
+	   (tae-compile-translation-element translation type decodep))
+	  ((eq op '|)
+	   (tae-compile-OR-operation args type decodep))
+	  ((eq op '&)
+	   (tae-compile-AND-operation args type decodep))
+	  ((eq op 'c)
+	   (tae-compile-composite-operation args type decodep))
+	  ((eq op 'ct)
+	   (tae-compile-composite-transparent-operation args type decodep))
+	  (t
+	   (error "TAE have not supported OP:`%S' yet!!" op)))))
 
 (defun tae-get-compiled-products (name type decodep)
   (let ((products (tae-get-translation-produced-products
-		   name mucs-current-type decodep))
-	tables-slot reduced-sym	translation)
+		   name type decodep))
+	table-set tables-slot translation)
+    ;; In order to make trasnaltion data persistent.
+    (tae-register-translation name)
+
     (if products
 	products
-      (setq reduced-sym (if decodep
-			    'tae-reduced-translation-for-decode
-			  'tae-reduced-translation-for-encode)
-	    translation (or (get name reduced-sym)
-			    (put name reduced-sym
-				 (tae-reduce-translation
-				  (symbol-value name)
-				  decodep)))
+      (setq translation (tae-reduce-translation
+			 name decodep)
 	    products (tae-get-compiled-products-internal
 		      translation type decodep))
 
@@ -1029,19 +1172,148 @@ according to it, rearrange the elements of UNIT."
       ;; convert it to table-set.
       (if (setq tables-slot (assq 'tables products))
 	  (progn
-	    (setcar tables-slot 'table-set)
-	    (setcdr tables-slot
-		    (tae-generate-new-table-set
-		     (cdr tables-slot) name decodep))))
+	    (if (setq table-set
+		      (tae-translation-get-table-set
+		       name type decodep))
+		(progn
+		  (define-table-set
+		    table-set
+		    (cdr tables-slot))
+		  table-set)
+	      (tae-generate-new-table-set
+	       (cdr tables-slot) name type decodep))
+	    (setq products
+		  (delq tables-slot products))))
       ;; register products.
-      (setq foo products)
       (tae-set-translation-produced-products
        name type decodep products)
       ;; return products.
       products)))
 
-;; API for compilation.
+;;;
+;;; Persistent data for translation rule.
+;;;   `Persistent' means data that is necessary
+;;; to manipulate the tranlation rule is kept in
+;;; byte-compiled file.
 
+(defun tae-generate-persistent-program (name persistent-items
+					     &optional subtype)
+  "Generate programs to make translation NAME persistent."
+  (if (not (tae-declared-translation-p name))
+      (error "NAME:%S is not declared translation rule." name))
+  (let (result current-item elem object)
+    (while (setq current-item (car persistent-items))
+      (setq persistent-items (cdr persistent-items))
+      (if (eq current-item 'tae-translation-definition)
+	  (and 
+	     ;;; See below note!!!
+;;	     (not (mucs-register-object current-item elem))
+	     (not (mucs-register-object name current-item))
+	     (setq result
+		   (cons
+		    `(setq ,name (quote ,(symbol-value name)))
+		    result)))
+	(setq elem
+	      (cond ((memq current-item
+			   '(tae-translation
+			     tae-dynamic-translation
+			     tae-dependent-conversions))
+		     (get name current-item))
+
+		    ((eq current-item
+			 'tae-reduced-translation-for-encode)
+		     (tae-reduce-translation name nil))
+		    ((eq current-item
+			 'tae-reduced-translation-for-decode)
+		     (tae-reduce-translation name t))
+
+		    ((memq current-item
+			   '(tae-products-for-encode
+			     tae-products-for-decode
+			     tae-table-set))
+		     (if subtype
+			 (tae-get-compiled-products
+			  name
+			  (if (eq current-item 
+				  'tae-products-for-decode)
+			      t nil)
+			  subtype)
+		       (get name current-item)))
+
+		    (t
+		     (error "%S is not valid item that can be persistent."
+			    current-item))))
+	(and elem
+	     ;;; Registering objects derived from translation-rule.
+	     ;;; The first line register objects generated by
+	     ;;; translation-rule as is.  On the contrary, the
+	     ;;; second line register symbol that identify generated
+	     ;;; objects.  It depends on object identification which one
+	     ;;; should be chosen.
+;;	     (not (mucs-register-object current-item elem))
+	     (not (mucs-register-object name current-item))
+	     (setq result
+		   (cons `(put (quote ,name)
+			       (quote ,current-item)
+			       (quote ,elem))
+			 result)))))
+;    (message "TR:%S,Result:%S" name result)
+    result
+;;; enclose with `progn' form?
+;    (if (> (length result) 1)
+;	(cons 'progn result)
+;      (car result))
+    ))
+
+(defun tae-compile-internal (name &optional decodep type)
+  (if (not (tae-declared-translation-p name))
+      (error "Undeclared or invalid translation:%S" name))
+  (let* ((tae-current-compiling-translation name)
+	 (messages (tae-get-compiled-products name type decodep))
+	 (mes-cur messages)
+	 (result-myo (mucs-ccl-empty-myo))
+	 (tae-translation-table-set-index
+	  tae-translation-table-set-index)
+	 mes-el
+	 table-set table-syms table-syms-nested-p)
+    ;; Mainly for dynamic modification,
+    ;; register dependent conversions.
+    (if mucs-current-conversion
+	(tae-register-conversion
+	 name type decodep
+	 mucs-current-conversion))
+    (while (setq mes-el (car mes-cur))
+      (setq mes-cur (cdr mes-cur))
+      (cond ((eq 'new-type (car mes-el))
+	     (setq mucs-current-type (cdr mes-el)))
+
+	    (t
+	     (error "Internal error.  unknown TAE message:%S" mes-el)
+	     )))
+    (setq table-set
+	  (tae-translation-get-table-set
+	   name type decodep)
+	  table-syms (get-table-set-symbol-list-recursively
+		      table-set)
+	  result-myo (mucs-ccl-myo-add-table-set
+		      table-set result-myo))
+    (if (table-set-nested-p table-set)
+	(setq table-syms-nested-p t))
+    (list
+     result-myo
+     table-syms
+     table-syms-nested-p)))
+
+(defun tae-compile-retrieve-ccl-prog (table-syms nestedp)
+  (if nestedp
+      `(loop
+	(r1 = 0)
+	(map-multiple r1 r0 ,table-syms))
+    `(loop
+      (r1 = 0)
+      (iterate-multiple-map r1 r0 ,@table-syms))))
+
+;; API for compilation.
 
 (defun tae-compile (name &optional decodep applied-type)
   "Retrieve MYO from the declared translation.
@@ -1050,45 +1322,64 @@ NAME must be a declared translation."
       (error "Undeclared or invalid translation:%S" name))
   (let* ((type (or applied-type
 		   mucs-current-type))
-	 (messages (tae-get-compiled-products name type decodep))
-	 (mes-cur messages)
-	 (result-myo (mucs-ccl-empty-myo))
-	 (tae-translation-table-set-index
-	  tae-translation-table-set-index)
-	 mes-el
-	 table-set table-syms table-syms-nested-p)
-    (while (setq mes-el (car mes-cur))
-      (setq mes-cur (cdr mes-cur))
-      (cond ((eq 'table-set (car mes-el))
-	     (if table-set
-		 (error
-		  "Internal error. TAE message cannot have more than 1 table-sets:%S"
-		  messages))
-	     (setq table-set (cdr mes-el)
-		   table-syms (get-table-set-symbol-list table-set)
-		   result-myo (mucs-ccl-myo-add-table-set
-			       table-set result-myo))
-	     (if (table-set-nested-p table-set)
-		 (setq table-syms-nested-p t)))
-
-	    ((eq 'new-type (car mes-el))
-	     (setq mucs-current-type (cdr mes-el)))
-
-	    (t
-	     (error "Internal error.  unknown TAE message:%S" mes-el)
-	     )))
-
-    (if table-syms-nested-p
-	(mucs-ccl-myo-add-ccl
-	 `((r1 = 0)
-	   (map-multiple r1 r0 ,table-syms))
-	 result-myo)
-      (mucs-ccl-myo-add-ccl
-       `((r1 = 0)
-	 (iterate-multiple-map r1 r0 ,@table-syms))
-       result-myo))
+	 (intdata (tae-compile-internal name decodep type))
+	 (result-myo (car intdata))
+	 (table-syms (nth 1 intdata))
+	 (table-syms-nested-p (nth 2 intdata)))
+    (mucs-ccl-myo-add-ccl
+     (list
+      (mucs-conversion-set-program-marker
+       (tae-generate-unique-symbol-for-embed
+	name type decodep)
+       (tae-compile-retrieve-ccl-prog
+	table-syms
+	table-syms-nested-p)))
+     result-myo)
     result-myo))
 
+;;; API for modification
+
+(defun tae-modify-translation (name rule)
+  "Modify declared tralslation.(Dynamic modification)"
+  (if (not (tae-translation-dynamic-p name))
+      (error "%S cannot be modified dynamically!"
+	     name))
+  (let ((conversion-list (get name 'tae-dependent-conversions))
+	(result-myo (mucs-ccl-empty-myo))
+	intdata	slot conv root-tr mark type decodep)
+    (set name rule)
+    (mapcar
+     (lambda (x)
+       (put name x nil))
+     '(tae-reduced-translation-for-encode
+       tae-reduced-translation-for-decode
+       tae-products-for-encode
+       tae-products-for-decode))
+    (while conversion-list
+      (setq slot (car conversion-list)
+	    conv (car slot)
+	    root-tr (nth 1 slot)
+	    mark (nth 2 slot)
+	    type (nth 3 slot)
+	    decodep (nth 4 slot)
+	    conversion-list (cdr conversion-list))
+
+      ;; reconstruct translation products.
+      (tae-compile-internal name decodep type)
+      ; flush generated products.
+;       (put root-tr
+; 	   (if decodep
+; 	       'tae-products-for-decode
+; 	     'tae-products-for-encode)
+; 	   nil)
+      (setq intdata (tae-compile-internal root-tr decodep type))
+      ;; modify conversion.
+      (mucs-modify-conversion
+       conv mark
+       (tae-compile-retrieve-ccl-prog
+	(nth 1 intdata)
+	(nth 2 intdata))))))
+
 ;; MAPPING FUNCTION
 ;;
 ;;  mapping functions are 
@@ -1104,7 +1395,8 @@ NAME must be a declared translation."
 	      funcs (cdr funcs))
 	(cond ((table-set-p elem)
 	       (setq ctables
-		     (get-table-set-symbol-list elem decodep)
+		     (get-table-set-symbol-list-recursively
+		      elem)
 		     tables (append tables ctables)))
 	       (t
 		(error "Not yet supported type! %S" elem))))
@@ -1125,13 +1417,15 @@ a list<CDR> (TABLE<SYMBOL> TABLE<SYMBOL> ...)."
 		 (setcdr tbl-c
 			 (nconc
 			  (cdr tbl-c)
-			  (get-table-set-tables elemf decodep)))
+			  (get-table-set-symbol-list-recursively
+			   elemf)))
 	       (setq tbl-alist
 		     (nconc
 		      tbl-alist
 		      (list
 		       (cons elemt
-			     (get-table-set-tables elemf decodep)))))))
+			     (get-table-set-symbol-list-recursively
+			      elemf)))))))
 	    (t
 	     (error "Not yet supported type! %S" elem))))
     (while (setq elem (car tbl-alist))
@@ -1166,4 +1460,35 @@ a list<CDR> (TABLE<SYMBOL> TABLE<SYMBOL> ...)."
 
 (defun tae-function-to-function (func1 reg))
 
+;;; generate registration program
+
+(defun tae-generate-registration-program ()
+  (let ((translation-list (tae-retrieve-registered-translation-list))
+	tr result)
+    (while (setq tr (car translation-list))
+      (setq result
+	    (append
+	     (tae-generate-persistent-program
+	      tr
+	      (if (tae-translation-dynamic-p tr)
+		  '(tae-translation-definition
+		    tae-translation
+		    tae-dynamic-translation
+		    tae-products-for-encode
+		    tae-products-for-decode
+		    tae-dependent-conversions
+		    tae-table-set)
+		'(tae-translation
+		  tae-products-for-encode
+		  tae-products-for-decode
+		  tae-table-set)))
+	     result)
+	    translation-list
+	    (cdr translation-list)))
+    result))
+
+;;; Add hook for embedding translation informations to a package.
+(add-hook 'mucs-package-definition-end-hook
+	  (function tae-generate-registration-program))
+
 (provide 'tae)
