@@ -1,6 +1,6 @@
 ;;; -*- byte-compile-dynamic: t -*-
 ;;; un-define.el --- fundamental template to comprise UTF
-;;;                  and Unicode conversion on Mule-UCS.
+;;;                  and Unicode conversions on Mule-UCS.
 
 ;; Copyright (C) 1999-2000 Miyashita Hisashi
 
@@ -89,22 +89,7 @@
     (mucs-ccl-stream-form
      (mucs-ccl-read 'char-packed-1 read)
      (tae-compile tr nil)
-     (cond ((eq write 'utf-7)
-	    (mucs-ccl-utf-7-encode
-	     'utf-7-direct-or-optional-character-p-translation
-	     nil))
-	   ((eq write 'utf-7-dos)
-	    (mucs-ccl-utf-7-encode
-	     'utf-7-direct-or-optional-character-p-translation
-	     t))
-	   ((eq write 'utf-7-safe)
-	    (mucs-ccl-utf-7-encode
-	     'utf-7-direct-character-p-translation nil))
-	   ((eq write 'utf-7-safe-dos)
-	    (mucs-ccl-utf-7-encode
-	     'utf-7-direct-character-p-translation t))
-	   (t
-	    (mucs-ccl-write write)))))
+     (mucs-ccl-write write)))
 
   (defun un-define-get-coding-system-alist
     (coding-category &optional mime-charset charsets)
@@ -466,6 +451,24 @@
   (ucs-generic . char-1)
   (0 ((tae-compile 'unicode-basic-non-stream-translation-rule t))))
 
+;;; For font encoder
+
+ (mucs-define-conversion
+  unicode-font-encoder
+  (font-codepoint . font-codepoint)
+  (0 ((mucs-ccl-font-encoder
+       (mapcar
+	(lambda (cs)
+	  (cons cs
+		`(list
+		  (mucs-ccl-convert-font-encoding
+		   ,(charset-dimension cs))
+		  (tae-compile 
+		   (quote
+		    ,(un-define-get-translation-from-charset cs)))
+		  (mucs-ccl-flat-code-to-font-encoding 2))))
+	un-define-basic-translation-charset-order-list)))))
+
 ;;
 ;; coding-system-definition.
 ;;
@@ -638,6 +641,18 @@
 
 ) ;; un-define package definition ends here
 
+
+;;; font encoder setup
+(add-to-list
+ 'font-ccl-encoder-alist
+ '("iso10646" . unicode-font-encoder))
+
+; font encoder setup (for Meadow)
+(if (featurep 'meadow)
+    (w32-regist-font-encoder
+     'unicode-font-encoder 'unicode-font-encoder))
+
+;;; coding-category setup
 (let (category-list)
 ;; set up coding-category
   (if (boundp 'coding-category-utf-8)
