@@ -28,6 +28,7 @@
 (require 'mucs)
 (require 'trans-util)
 (require 'tae)
+(require 'mccl-font)
 
 (require 'un-data)
 
@@ -154,15 +155,16 @@ If you want to deal with untranslated character, use this translation rule.")
 (mucs-type-register-serialization
  'ucs-generic
  'utf-8
- utf-8-ccl-encode
- utf-8-ccl-decode)
+ (quote `(,utf-8-ccl-encode))
+ (quote `(,utf-8-ccl-decode)))
 
 (mucs-type-register-serialization
  'ucs-generic
  'utf-8-dos
- (append
-  (convert-unicode-lf-2-crlf '((write ?\xd)))
-  utf-8-ccl-encode)
+ (quote
+  `(,(append
+      (convert-unicode-lf-2-crlf '((write ?\xd)))
+      utf-8-ccl-encode)))
  'none)
 
 (defun mucs-ccl-write-utf-8-signature ()
@@ -235,15 +237,16 @@ If you want to deal with untranslated character, use this translation rule.")
 (mucs-type-register-serialization
  'ucs-generic
  'utf-16-le
- utf-16-le-ccl-encode
- utf-16-le-ccl-decode)
+ (quote `(,utf-16-le-ccl-encode))
+ (quote `(,utf-16-le-ccl-decode)))
 
 (mucs-type-register-serialization
  'ucs-generic
  'utf-16-le-dos
- (append
-  (convert-unicode-lf-2-crlf '((write "\x0D\x00")))
-  utf-16-le-ccl-encode)
+ (quote
+  `(,(append
+      (convert-unicode-lf-2-crlf '((write "\x0D\x00")))
+      utf-16-le-ccl-encode)))
  'none)
 
 (defun mucs-ccl-write-utf-16-le-signature ()
@@ -271,15 +274,17 @@ If you want to deal with untranslated character, use this translation rule.")
 (mucs-type-register-serialization
  'ucs-generic
  'utf-16-be
- utf-16-be-ccl-encode
- utf-16-be-ccl-decode)
+ (quote `(,utf-16-be-ccl-encode))
+ (quote `(,utf-16-be-ccl-decode)))
 
 (mucs-type-register-serialization
  'ucs-generic
  'utf-16-be-dos
- (append
-  (convert-unicode-lf-2-crlf '((write "\x00\x0D")))
-  utf-16-be-ccl-encode)
+ (quote
+  `(,(append
+      (convert-unicode-lf-2-crlf
+       '((write "\x00\x0D")))
+      utf-16-be-ccl-encode)))
  'none)
 
 (defun mucs-ccl-write-utf-16-be-signature ()
@@ -318,6 +323,27 @@ If you want to deal with untranslated character, use this translation rule.")
   '(assoc (ucs-generic . ucs-generic)
 	  ((all . invalid))))
 
+; (defconst utf-7-b64-assoc
+;   `(assoc
+;     (b64-char . number)
+;     '((?A . 00) (?B . 01) (?C . 02) (?D . 03)
+;       (?E . 04) (?F . 05) (?G . 06) (?H . 07)
+;       (?I . 08) (?J . 09) (?K . 10) (?L . 11)
+;       (?M . 12) (?N . 13) (?O . 14) (?P . 15)
+;       (?Q . 16) (?R . 17) (?S . 18) (?T . 19)
+;       (?U . 20) (?V . 21) (?W . 22) (?X . 23)
+;       (?Y . 24) (?Z . 25) (?a . 26) (?b . 27)
+;       (?c . 28) (?d . 29) (?e . 30) (?f . 31)
+;       (?g . 32) (?h . 33) (?i . 34) (?j . 35) 
+;       (?k . 36) (?l . 37) (?m . 38) (?n . 39) 
+;       (?o . 40) (?p . 41) (?q . 42) (?r . 43) 
+;       (?s . 44) (?t . 45) (?u . 46) (?v . 47)
+;       (?w . 48) (?x . 49) (?y . 50) (?z . 51)
+;       (?0 . 52) (?1 . 53) (?2 . 54) (?3 . 55)
+;       (?4 . 56) (?5 . 57) (?6 . 58) (?7 . 59)
+;       (?8 . 60) (?9 . 61) (?+ . 62) (?/ . 63)
+;       (all . invalid) (invalid . all))))
+
 (defconst ccl-b64-encode-table
   '[?A ?B ?C ?D ?E ?F ?G ?H ?I ?J ;; 0-9
     ?K ?L ?M ?N ?O ?P ?Q ?R ?S ?T ;; 10-19
@@ -420,26 +446,26 @@ If you want to deal with untranslated character, use this translation rule.")
 	,(ccl-utf-7-base64-encode-flush 'r6))))
 
 (defun mucs-ccl-utf-7-encode (direct-character-translation dos)
-  (mucs-ccl-bind-program
-   `(,(mucs-ccl-if-invalid-repeat)
-     ((r4 = r0))
-     ,(tae-compile direct-character-translation nil)
-     ,(mucs-ccl-if-invalid
+  `(((r4 = r0))
+    ,(tae-compile direct-character-translation nil)
+    ,(mucs-ccl-if-invalid
        ;;; shifted state
-       `((if ,(mucs-ccl-check-internal-state 'utf-7-shifted-p)
-	     ,(ccl-utf-7-base64-encode 'r4 'r6)
-	   ((write ?+)
-	    ,@(ccl-utf-7-base64-encode 'r4 'r6 t))))
+      (quote
+       `(((if ,(mucs-ccl-check-internal-state 'utf-7-shifted-p)
+	      ,(ccl-utf-7-base64-encode 'r4 'r6)
+	    ((write ?+)
+	     ,@(ccl-utf-7-base64-encode 'r4 'r6 t))))))
        ;;; Non-shifted state
-       `((if ,(mucs-ccl-check-internal-state 'utf-7-shifted-p)
-	     ,(ccl-utf-7-base64-encode-flush 'r6))
-	 ,@(if dos
-	       '((if (r0 == ?\n)
-		     ((write "\x0D\x0A"))
-		   ((write r0))))
-	     '((write r0)))
-	 (if (r0 == ?+)
-	     ((write ?-))))))))
+      (quote
+       `(((if ,(mucs-ccl-check-internal-state 'utf-7-shifted-p)
+	      ,(ccl-utf-7-base64-encode-flush 'r6))
+	  ,@(if dos
+		'((if (r0 == ?\n)
+		      ((write "\x0D\x0A"))
+		    ((write r0))))
+	      '((write r0)))
+	  (if (r0 == ?+)
+	      ((write ?-)))))))))
 
 (defconst ccl-utf-7-decode
   `((r4 = 0)
@@ -502,8 +528,34 @@ If you want to deal with untranslated character, use this translation rule.")
 (mucs-type-register-serialization
  'ucs-generic
  'utf-7
- 'none
- ccl-utf-7-decode)
+ '(mucs-ccl-utf-7-encode
+   'utf-7-direct-or-optional-character-p-translation
+   nil)
+ (quote `(,ccl-utf-7-decode)))
+
+(mucs-type-register-serialization
+ 'ucs-generic
+ 'utf-7-safe
+ '(mucs-ccl-utf-7-encode
+   'utf-7-direct-character-p-translation
+   nil)
+ 'none)
+
+(mucs-type-register-serialization
+ 'ucs-generic
+ 'utf-7-dos
+ '(mucs-ccl-utf-7-encode
+   'utf-7-direct-or-optional-character-p-translation
+   t)
+ 'none)
+
+(mucs-type-register-serialization
+ 'ucs-generic
+ 'utf-7-safe-dos
+ '(mucs-ccl-utf-7-encode
+   'utf-7-direct-character-p-translation
+   t)
+ 'none)
 
 ;
 ; Assoc translation rule loader
